SystemOrganization addCategory: #'GT-Examples'!
SystemOrganization addCategory: 'GT-Examples-Core'!
SystemOrganization addCategory: 'GT-Examples-Exceptions'!
SystemOrganization addCategory: 'GT-Examples-TODO'!
SystemOrganization addCategory: 'GT-Examples-Testing'!
SystemOrganization addCategory: 'GT-Examples-UI'!
SystemOrganization addCategory: 'GT-Examples-Utils'!

!Dictionary methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 21:38'!
removeKey: aKey thenDo: aBlock
	aBlock value: (self removeKey: aKey ifAbsent: [ ^ self ])! !

!RPackageTag methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:49'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self methods do: [ :method |
		method gtExamplesFactory exampleMethod: method do: [ :example | 
			examples add: example ] ].
	^ examples! !

!ExceptionSet methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 6/15/2015 16:56'!
copyWith: anException
	^ self class new 
		instVarNamed: #exceptions put: (exceptions copyWith: anException);
		yourself! !

!ExceptionSet methodsFor: '*GT-Examples-Core' stamp: 'AndreiChis 10/15/2015 00:15'!
copyWithAll: aCollection
	^ self class new 
		instVarNamed: #exceptions put: (exceptions copy
			addAll: aCollection;
			yourself);
		yourself! !

!ExceptionSet methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/6/2015 17:43'!
signaledBy: anException
	(anException isNil and: [ exceptions isEmptyOrNil not ]) ifTrue: [ 
		^ (GTExampleExpectedExceptionNotRaised new 
			expectedError: self;
			exception: anException;
			yourself) signal ].
	(anException notNil and: [ (exceptions includes: anException class) not ]) ifTrue: [ 
		^ (GTExampleUnexpectedExceptionRaised new
			unexpectedError: anException;
			yourself) signal ]! !

Object subclass: #GTAbstractExampleFactory
	instanceVariableNames: 'cache source providers exampleClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 20:53'!
cache
	^ cache ifNil: [ cache := self defaultCache ]! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:16'!
cache: anObject
	^ cache := anObject! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 19:34'!
createExample: aCompiledMethod
	| example |
	example := self exampleClass new.
	self initializeFromMethod: aCompiledMethod forExample: example.
	^example! !

!GTAbstractExampleFactory methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 7/3/2016 16:28'!
defaultCache
	^ GTExampleOrganizer instance! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 19:18'!
exampleAt: aCompiledMethod ifAbsentPut: aBlock
	^ aBlock value! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:16'!
exampleClass
	^ exampleClass ifNil: [ exampleClass := self defaultExampleClass ]! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:16'!
exampleClass: anObject
	exampleClass := anObject! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 19:18'!
exampleFromMethod: aCompiledMethod do: aBlock
	aBlock value: (self cache exampleAt: aCompiledMethod ifAbsentPut: [ self createExample: aCompiledMethod ])! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 20:48'!
exampleMethod: aCompiledMethod do: aBlock
	(self isExampleMethod: aCompiledMethod) ifFalse: [ ^ self ].
	self exampleFromMethod: aCompiledMethod do: aBlock! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 20:21'!
exampleMethodsDo: aBlock
	self source theNonMetaClass methods
		select: [ :each | self isExampleMethod: each ]
		thenDo: aBlock.
	self source theMetaClass methods
		select: [ :each | self isExampleMethod: each ]
		thenDo: aBlock.
		! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 22:07'!
examplesDo: aBlock
	self exampleMethodsDo: [ :method |
		self exampleFromMethod: method do: aBlock ]! !

!GTAbstractExampleFactory methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 20:23'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self examplesDo: [ :example | examples add: example ].
	^ examples! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 12:54'!
initializeFromMethod: aCompiledMethod forExample: aGTExample
	aGTExample methodClass: aCompiledMethod methodClass.
	aGTExample selector: aCompiledMethod selector.! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 12:22'!
initializePragmasFromMethod: aCompiledMethod forExample: aGTExample
	aCompiledMethod pragmas do: [ :pragma | 
		(aGTExample class canPerform: pragma keyword) ifTrue: [ 
			[ aGTExample perform: pragma keyword withArguments: pragma arguments ]
				on:Error
				do:[ :exception | 
					aGTExample failures add: (exception asGTExamplePragmaError 
						example: aGTExample;
						pragma: pragma;
						yourself) ] ] ]! !

!GTAbstractExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 12:16'!
initializeSubjectsFromClass: aClass forExample: aGTExample
	aClass gtExamplesSubjects do: [ :subject | 
		[ aGTExample subjects add: subject asGTExampleSubject ]
			on:Error
			do: [ :exception | 
				aGTExample failures add: (GTExampleSubjectError new 
					example: aGTExample;
					subject: subject;
					signaledException: exception;
					yourself) ] ] ! !

!GTAbstractExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 19:18'!
isExampleMethod: aCompiledMethod
	self subclassResponsibility! !

!GTAbstractExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:38'!
isGTExampleMethod: aCompiledMethod
	^ aCompiledMethod isGTExampleMethod! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:41'!
provider
	^ provider ifNil: [ provider := self defaultProvider ]! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:41'!
provider: anObject
	provider := anObject! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 18:58'!
source
	^ source! !

!GTAbstractExampleFactory methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 18:58'!
source: anObject
	source := anObject! !

GTAbstractExampleFactory subclass: #GTExampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleFactory methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 7/2/2016 19:16'!
defaultExampleClass
	^ GTExample! !

!GTExampleFactory methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 19:34'!
initializeFromMethod: aCompiledMethod forExample: aGTExample
	super initializeFromMethod: aCompiledMethod forExample: aGTExample.
	
	self initializeSubjectsFromClass: source forExample: aGTExample.
	self initializePragmasFromMethod: aCompiledMethod forExample: aGTExample.
! !

!GTExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 20:00'!
isExampleMethod: aCompiledMethod
	^ self isGTExampleMethod: aCompiledMethod ! !

GTAbstractExampleFactory subclass: #RTExampleFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!RTExampleFactory methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 7/2/2016 19:54'!
defaultExampleClass
	^ RTExampleSelection! !

!RTExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 20:01'!
isExampleMethod: aCompiledMethod
	^ (self isGTExampleMethod: aCompiledMethod)
		or: [ self isOldStyleRTExampleMethod: aCompiledMethod ]! !

!RTExampleFactory methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 20:01'!
isOldStyleRTExampleMethod: aCompiledMethod
	^ (aCompiledMethod selector beginsWith: 'example')
		and: [ (aCompiledMethod selector endsWith: 'Icon') not ]! !

Object subclass: #GTAbstractExampleSubject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTAbstractExampleSubject methodsFor: 'converting' stamp: 'StefanReichhart 7/2/2016 19:45'!
asGTExampleSubject
	^ self! !

!GTAbstractExampleSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 18:45'!
hasLiteralThorough: aLiteral
	^ false! !

GTAbstractExampleSubject subclass: #GTExampleClassSubject
	instanceVariableNames: 'subjectClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleClassSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/2/2016 19:45'!
exists
	^ self subclassResponsibility! !

!GTExampleClassSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 00:23'!
hasLiteralThorough: aLiteral
	^ self subjectClass = aLiteral
		or: [ self subjectClass name = aLiteral
			or: [ self subjectClass methods anySatisfy: [ :method | 
						method selector = aLiteral
							or: [ 	method hasLiteralThorough: aLiteral ] ] ] ]! !

!GTExampleClassSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectClass
	^ subjectClass! !

!GTExampleClassSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectClass: anObject
	subjectClass := anObject! !

GTAbstractExampleSubject subclass: #GTExampleInstanceSubject
	instanceVariableNames: 'instance'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleInstanceSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 18:49'!
hasLiteralThorough: aLiteral
	^ (self instance class name = aLiteral)
		or: [ self instance class methods anySatisfy: [ :method | 
					method selector = aLiteral
						or: [ 	method hasLiteralThorough: aLiteral ] ] ]! !

!GTExampleInstanceSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:42'!
instance
	^ instance! !

!GTExampleInstanceSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:42'!
instance: anObject
	instance := anObject! !

GTAbstractExampleSubject subclass: #GTExampleMethodSubject
	instanceVariableNames: 'subjectClass subjectSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleMethodSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 00:23'!
hasLiteralThorough: aLiteral
	^ subjectSelector = aLiteral
		or: [ self hasMethod
			and: [ self method hasLiteralThorough: aLiteral ] ]! !

!GTExampleMethodSubject methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 18:46'!
hasMethod
	^ self subjectClass includesSelector: self subjectSelector! !

!GTExampleMethodSubject methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 18:47'!
method
	^ self subjectClass >> self subjectSelector! !

!GTExampleMethodSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectClass
	^ subjectClass! !

!GTExampleMethodSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectClass: anObject
	subjectClass := anObject! !

!GTExampleMethodSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectSelector
	^ subjectSelector! !

!GTExampleMethodSubject methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:40'!
subjectSelector: anObject
	subjectSelector := anObject! !

Object subclass: #GTExample
	instanceVariableNames: 'methodClass selector label description form exceptions subjects children properties after problems'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Core'!

!GTExample commentStamp: 'TudorGirba 9/6/2015 11:03' prior: 0!
I represent the concept of an example.

I am constructed out of a method that returns an object. I hold the meta-information about that method and I know how to construct the concrete resulting object that is returned by the method. 

I can depend on other examples, and other examples can depend on me. When I depend on another example, the code that produces me uses the result of the other example to construct me.

The class for which I represent the example is called my subject.

The class that offers the method out of which I am constructed is the provider.

I also know how to handle possible exceptions, which makes me a great candidate for encoding assumptions that are otherwise expressed as classic tests.!

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 12:56'!
= anExample
	^ self class = anExample class 
		and: [ self methodClass = anExample methodClass 
			and: [ self selector = anExample selector ] ]! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 20:44'!
after
	^ after! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 4/2/2016 18:56'!
after: aSelector
	<gtExamplePragma>
	<description: 'A method, provided by the given selector, to be performed after this example has run ~ similar but not identical to teardown of sunit'>
	
	after := self methodForSelector: aSelector! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 4/10/2016 18:59'!
afterArguments
	self after ifNil: [ ^ #() ].
	^ self after arguments! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 20:56'!
afterClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'A method (implemented by a gtExampleProvider) to be performed after this example has run ~ similar but not identical to teardown of sunit'>
	
	after := self methodForClassNamed: aClassOrClassName meta: false selector: aSelector! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 20:56'!
afterMetaClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'A method (implemented by a gtExampleProvider) to be performed after this example has run ~ similar but not identical to teardown of sunit'>
	
	after := self methodForClassNamed: aClassOrClassName meta: true selector: aSelector! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/24/2015 21:38'!
allDependencies
	| examples |
	examples := Set new.
	(self traverser
		action: [ :example | examples add: example ];
		yourself) value.
	^ examples! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/24/2015 21:35'!
allDependents
	| examples |
	examples := Set new.
	(self traverser reversed
		action: [ :example | examples add: example ];
		yourself) value.
	^ examples! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/24/2015 20:54'!
arguments
	^ children asArray! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/29/2015 11:03'!
classNamed: aClassOrClassName
	^ aClassOrClassName isClass 
		ifTrue: [ aClassOrClassName ]
		ifFalse: [ Smalltalk classNamed: aClassOrClassName asString ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 17:44'!
classNamed: aClassName do: aBlock
	^ self classNamed: aClassName do: aBlock ifNone: [ 
		(GTExampleDeclaredClassMissing new
			example: self;
			classNamed: aClassName;
			yourself) signal ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 17:04'!
classNamed: aClassName do: aBlock1 ifNone: aBlock2
	^ (self classNamed: aClassName)
		ifNotNil: aBlock1
		ifNil: aBlock2! !

!GTExample methodsFor: 'actions' stamp: 'StefanReichhart 9/2/2015 22:19'!
debug
	^ self debugger result! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 17:42'!
debugger
	^ GTExampleDebugger new 
		example: self;
		yourself! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 15:02'!
dependencies
	^ self arguments collect: [ :each | each gtExample ]! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 15:27'!
dependenciesOrArguments
	^ self arguments collect: [ :each | each gtExample ifNil: [ each ] ]! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 13:41'!
depends: aSelector
	<gtExamplePragma>
	<description: 'This example depends on the example declared by the given selector. Any dependency will be performed before this example is performed. Order of declaration is important'>
	
	children add: (self methodForSelector: aSelector)! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:31'!
dependsAll: aString
	<gtExamplePragma>
	<description: 'This example depends on all given selectors found in the given string. Any dependency will be performed before this example is performed. Order of declaration is important'>
	
	self substrings: aString do: [ :each | 
		self depends: aString ]! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 20:55'!
dependsClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'This example depends on the example declared by the given class and selector. Any dependency will be performed before this example is performed. Order of declaration is important'>
	
	children add: (self methodForClassNamed: aClassOrClassName meta: false selector: aSelector)! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 21:02'!
dependsMetaClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'This example depends on the example declared by the given class and selector. Any dependency will be performed before this example is performed. Order of declaration is important'>
	children add: (self methodForClassNamed: aClassOrClassName meta: true selector: aSelector)! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 6/7/2015 17:39'!
description
	^ description! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:28'!
description: anObject
	<gtExamplePragma>
	<description: 'A full description of this example'>
	
	description := anObject! !

!GTExample methodsFor: 'TODO' stamp: 'StefanReichhart 7/3/2016 17:11'!
directDependents
	<todo: 'better name -> #myDependents and #dependents is taken - wtf!!'>
	^ (GTExampleOrganizer instance examplesSelect: [ :each | each hasDependency: self ]) asSet! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 17:42'!
evaluator
	^ GTExampleEvaluator new 
		example: self;
		yourself! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/29/2015 10:58'!
exceptions
	^ exceptions! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:31'!
exists
	^ self methodClass includesSelector: self selector! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 16:10'!
gtDebuggerSourceIn: composite
	<gtDebuggerPresentationOrder: 1>
	self hasMethod ifFalse: [ ^ self ].
	^ (self method gtInspectorSourceIn: composite)
		title: 'Method'! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 8/2/2015 14:18'!
gtDisplayOn: aStream
	self printOn: aStream.
	aStream nextPutAll: ' ('.
	self gtDisplaySummaryOn: aStream.
	aStream nextPutAll: ')'! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 16:06'!
gtDisplaySummaryOn: aStream.
	self label isEmptyOrNil 
		ifFalse: [ ^ aStream nextPutAll: self label ].
	self description isEmptyOrNil 
		ifFalse: [ ^ aStream nextPutAll: self description ].
	aStream 
		print: self methodClass; 
		nextPutAll: '>>'; 
		print: self selector.! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:28'!
gtExample
	<gtExamplePragma>
	<description: 'This method is turned into a gtExample if provided'>
	
	^ self! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 15:03'!
gtExample: anObject
	<toBeDeprecated: 'Please define your example-label (or description) using the pragma #label: or #description:'>
	self label: anObject.
	self gtExample! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 15:19'!
gtExamplesContained
	^ { self }! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 15:02'!
gtExamplesFactory
	^ self method gtExamplesFactory! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 15:29'!
gtInspectorActionEvaluateResultAndGo
	<gtInspectorAction>
	^ GLMGenericAction new
		action: [ :p | p selection: self returnValue ];
		icon: (Smalltalk ui icons iconNamed: #glamorousRight);
		title: 'Evaluate result and go'! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 15:29'!
gtInspectorDependenciesIn: composite
	<gtInspectorPresentationOrder: 2>
	self hasDependencies ifFalse: [ ^ self ].
	^ (self dependenciesOrArguments gtInspectorItemsIn: composite)
			title: 'Dependencies'! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 16:45'!
gtInspectorDependentsIn: composite
	<gtInspectorPresentationOrder: 3>
	self hasDependents ifFalse: [ ^ self ].
	^ (self directDependents gtInspectorItemsIn: composite)
			title: 'Dependents'! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 16:10'!
gtInspectorMethodIn: composite
	<gtInspectorPresentationOrder: 0>
	self hasMethod ifFalse: [ ^ self ].
	^ (self method gtInspectorSourceIn: composite)
		title: 'Method'! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 9/21/2015 20:35'!
gtInspectorRoassalIn: composite
	<gtInspectorPresentationOrder: 1>
	self hasRelationships ifFalse: [ ^ self ].
	^ self painter gtInspectorRoassalIn: composite! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 15:27'!
gtSpotterDependenciesFor: aStep
	<spotterOrder: 100>
	self hasDependencies ifFalse: [ ^ self ].
	aStep listProcessor
		title: 'Example Dependencies';
		allCandidates: [ self dependenciesOrArguments ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 16:45'!
gtSpotterDependentsFor: aStep
	<spotterOrder: 101>
	self hasDependents ifFalse: [ ^ self ].
	aStep listProcessor
		title: 'Example Dependents';
		allCandidates: [ self directDependents ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 16:10'!
gtSpotterMethodFor: aStep
	<spotterOrder: 103>
	self hasMethod ifFalse: [ ^ self ].
	aStep listProcessor
		title: 'Method';
		items: [ Array with: self method ];
		itemName: [ :aSource | aSource gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 15:25'!
gtSpotterPreviewIn: aComposite
	<spotterPreview: 1>
	self hasMethod ifFalse: [ ^ self ].
	aComposite pharoMethod
		title: [ self gtDisplayString ]; 
		display: [ :entity | entity method sourceCode ];
		smalltalkClass: [ self method methodClass ];
		entity: self! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 15:25'!
gtSpotterSubjectsFor: aStep
	<spotterOrder: 102>
	self hasSubjects ifFalse: [ ^ self ].
	aStep listProcessor
		title: 'Example Subjects';
		allCandidates: [ self subjects ];
		itemName: [ :subject | subject gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:09'!
hasAfter
	^ after notNil! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:09'!
hasAfterArguments
	^ self hasAfter
		and: [ self after hasArguments ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:06'!
hasArguments
	^ self arguments notEmpty! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:04'!
hasDependencies
	^ self arguments notEmpty! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:58'!
hasDependency: aGTExample
	^ self arguments anySatisfy: [ :each | each matchesExample: aGTExample ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 16:45'!
hasDependent: anExample
	^ self directDependents anySatisfy: [ :each | each matchesExample: anExample ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 16:45'!
hasDependents
	^ self directDependents notEmpty! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:26'!
hasMethod
	^ self exists! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 00:12'!
hasProblems
	^ problems notEmpty
		or: [ self hasValidArguments not
			or: [ self hasValidAfterArguments not 
				or: [ self hasValidDependencies not ] ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 9/21/2015 20:33'!
hasRelationships
	^ self hasDependents
		or: [ self hasDependencies ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 00:21'!
hasSubject: aSubject
	| subject |
	subject := self classNamed: aSubject do: [ :class | class ] ifNone: [ aSubject ].
	^ self subjects anySatisfy: [ :each | each hasLiteralThorough: subject ]
	
	! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:26'!
hasSubjects
	^ self subjects notEmpty! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:10'!
hasValidAfterArguments
	^ self hasAfterArguments not
		or: [ self after hasValidArguments ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:07'!
hasValidArguments
	^ self arguments size = self method numArgs! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 20:32'!
hasValidDependencies
	^ self hasValidArguments 
		and: [ self hasDependencies not
			or: [ self dependencies allSatisfy: [ :each | 
				each notNil and: [ each exists ] ] ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 12:56'!
hash
	^ (self class hash 
		bitXor: self methodClass hash) 
			bitXor: self selector hash! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 9/4/2015 16:51'!
icon
	^ form! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:09'!
icon: aSelector
	<gtExamplePragma>
	<description: 'This is a shortcut for: #iconThemeSelector:'>
	
	self iconThemeSelector: aSelector! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:10'!
iconBase64: aString
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The string must represent a base64 string'>
	
	form := Form fromBinaryStream: (Base64MimeConverter mimeDecodeToBytes: aString readStream)! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:10'!
iconBytes: aByteArray
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The icon form is described by the given byte-array'>
	
	form := (Form
		extent: 16@16
		depth: 32
		fromArray: aByteArray 
		offset: 0@0)! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:10'!
iconClass: aClassOrClassName selector: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The method described by the given class and selector must return a form-icon'>
	
	self classNamed: aClassOrClassName do: [ :class | 
		form := class perform: aSelector ]! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 14:45'!
iconSelector: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The selector must return an icon form'>
	
	form := self provider perform: aSelector! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:09'!
iconSelectorBase64: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The selector must return a base64 string'>
	
	self iconBase64: (self provider perform: aSelector)! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:09'!
iconThemeSelector: aSelector
	<gtExamplePragma>
	<description: 'Attach an icon to this example. The selector must be implemented by the currently active theme'>
	
	form := Smalltalk ui icons perform: aSelector! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 9/26/2015 10:59'!
ignoreTest
	^ self propertyAt: #noTest ifAbsent: [ false ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 4/24/2016 17:16'!
ignoreTestReport
	^ self propertyAt: #noTestReport ifAbsent: [ false ]! !

!GTExample methodsFor: 'initializing' stamp: 'StefanReichhart 7/3/2016 23:01'!
initialize
	super initialize.
	
	children := OrderedCollection new.
	subjects := Set new.
	problems := Set new.
	exceptions := ExceptionSet new.	
	properties := IdentityDictionary new.! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 9/6/2015 17:57'!
isGTExample
	^ true! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 08:41'!
isReferencing: aLiteral
	^ (self hasSubject: aLiteral)
		or: [ (self subjects anySatisfy: [ :subject | subject hasLiteralThorough: aLiteral ])
			or: [ (self hasMethod and: [ self method hasLiteralThorough: aLiteral ])
				or: [ (self methodClass name asString = aLiteral name asString) ] ] ]! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 22:53'!
isValid
	^ self hasProblems not
		and: [ self traverser canProcess ]! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 6/7/2015 17:39'!
label
	^ label! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:28'!
label: anObject
	<gtExamplePragma>
	<description: 'A short identifier of the name of this example'>
	
	label := anObject! !

!GTExample methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:30'!
matchesMethod: aCompiledMethod
	^ aCompiledMethod class = CompiledMethod
		and: [ self methodClass = aCompiledMethod methodClass
			and: [ self selector = aCompiledMethod selector ] ]! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 12:34'!
method
	^ self methodClass >> self selector! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:27'!
methodClass
	^ methodClass! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:27'!
methodClass: aClass
	methodClass := aClass! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 20:43'!
methodForClass: aClass named: aClassName meta: aBoolean selector: aSelector
	^ GTExampleMethod new
		methodClass: aClass;
		methodClassName: aClassName;
		meta: aBoolean;
		selector: aSelector asSymbol;
		yourself! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 20:43'!
methodForClass: aClass selector: aSelector
	^ self methodForClass: aClass named: aClass name meta: aClass isMeta selector: aSelector! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 21:01'!
methodForClassNamed: aClassOrClassName meta: aBoolean selector: aSelector
	^ self classNamed: aClassOrClassName 
		do: [ :class | self methodForClass: (aBoolean ifTrue: [ class class ] ifFalse: [ class ]) named: class name meta: aBoolean selector: aSelector ] 
		ifNone: [ self methodForClass: nil named: aClassOrClassName meta: aBoolean selector: aSelector ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 20:57'!
methodForSelector: aSelector
	^ self methodForClass: self methodClass selector: aSelector! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 15:19'!
methodReference
	^ self method asRingDefinition! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:26'!
noTest
	<gtExamplePragma>
	<description: 'Do not run this example while running SUnit tests'>
	
	self propertyAt: #noTest put: true! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 4/24/2016 17:17'!
noTestReport
	<gtExamplePragma>
	<description: 'Do not run this example while running HDTestReport (not yet implemented)'>
	
	self propertyAt: #noTestReport put: true! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 9/6/2015 17:45'!
painter
	^ GTExamplePainter new 
		example: self;
		yourself! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 22:52'!
problems
	^ problems! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 9/26/2015 10:58'!
propertyAt: aKey
	^ (properties ifNil: [ KeyNotFound signalFor: aKey ])
		at: aKey! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 9/26/2015 10:56'!
propertyAt: aKey ifAbsent: aBlock
	^ (properties ifNil: [ ^ aBlock value ])
		at: aKey ifAbsent: aBlock! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 9/26/2015 10:57'!
propertyAt: aKey ifAbsentPut: aBlock
	^ (properties ifNil: [ properties := IdentityDictionary new ])
		at: aKey ifAbsentPut: aBlock! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 9/26/2015 10:56'!
propertyAt: aKey put: aValue
	^ (properties ifNil: [ properties := IdentityDictionary new ])
		at: aKey put: aValue! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 12:54'!
provider 
	^ self methodClass isMeta
		ifTrue: [ self methodClass theNonMetaClass ]
		ifFalse: [ self methodClass theNonMetaClass new ]! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:26'!
raises: aClassOrClassName
	<gtExamplePragma>
	<description: 'This example is expected to raise the given exception. The example will fail if the exception is not raised.'>
	
	self classNamed: aClassOrClassName do: [ :class | 
		self exceptions add: class ]! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:26'!
raisesAny: aString
	<gtExamplePragma>
	<description: 'This example is expected to raise any of the given exceptions. The example will fail if none of the exceptions is raised.'>
	
	self substrings: aString do: [ :each | 
		self raises: each ]! !

!GTExample methodsFor: 'initializing' stamp: 'StefanReichhart 7/3/2016 12:34'!
reinitialize
	| method |
	(method := self method) gtExamplesFactory 
		initializeFromMethod: method forExample: self! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 19:05'!
result
	^ self run! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 8/2/2015 12:00'!
returnValue
	^ self result returnValue! !

!GTExample methodsFor: 'actions' stamp: 'StefanReichhart 7/2/2016 21:55'!
run
	^ self evaluator result ! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:27'!
selector
	^ selector! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:27'!
selector: aSelector
	selector := aSelector! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 5/13/2016 11:02'!
signalableExceptions
	^ self exceptions copyWithAll: { Halt. Error. Exit. UnhandledError. TestFailure. Deprecation. }! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 18:37'!
sourceCode
	^ self method sourceCode! !

!GTExample methodsFor: 'ui' stamp: 'StefanReichhart 9/9/2015 20:10'!
spotterPreviewIn: aComposite inContext: aSpotter
	^ self gtSpotterPreviewIn: aComposite! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 7/3/2016 12:30'!
subject: aClassOrClassName
	<gtExamplePragma>
	
	self classNamed: aClassOrClassName do: [ :class | 
		self subjects add: class asGTExampleSubject ]! !

!GTExample methodsFor: 'accessing' stamp: 'StefanReichhart 7/29/2015 11:00'!
subjects
	^ subjects! !

!GTExample methodsFor: 'pragma-selectors' stamp: 'StefanReichhart 3/28/2016 19:39'!
subjectsAll: aString
	<gtExamplePragma>
	
	self substrings: aString do: [ :each | 
		self subject: each ]! !

!GTExample methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 17:09'!
substrings: aString do: aBlock
	(aString substrings: ' .,;') do: [ :each | 
		aBlock value: each trimBoth ]! !

!GTExample methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 9/6/2015 17:42'!
traverser
	^ GTExampleTraverser new 
		example: self;
		yourself! !

Object subclass: #GTExampleMethod
	instanceVariableNames: 'methodClassName methodClass meta selector arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 20:46'!
= anExampleMethod
	^ self class = anExampleMethod class 
		and: [ self methodClassName = anExampleMethod methodClassName 
			and: [ self methodClass = anExampleMethod methodClass
				and: [ self meta = anExampleMethod meta
					and: [ self selector = anExampleMethod selector ] ] ] ]! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:23'!
arguments
	^ arguments ifNil: [ #() ]! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:23'!
arguments: anObject
	arguments := anObject! !

!GTExampleMethod methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 20:45'!
ensure
	methodClass 
		ifNil: [ methodClass := self meta 
			ifTrue: [ (Smalltalk classNamed: self methodClassName) class ] 
			ifFalse: [ Smalltalk classNamed: self methodClassName ] ]
		ifNotNil: [ methodClass isObsolete ifTrue: [ methodClass := nil ] ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 20:47'!
exists
	^ self methodClassName notNil
		and: [ self methodClass notNil
			and: [ self methodClass includesSelector: self selector ] ]! !

!GTExampleMethod methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 13:39'!
gtExample
	^ self hasExample 
		ifTrue: [ self method gtExample ]
		ifFalse: [ nil ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/4/2016 00:13'!
hasArguments
	^ self arguments notEmpty! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 16:13'!
hasExample
	^ self exists
		and: [ self method isGTExampleMethod ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 15:07'!
hasValidArguments
	^ self exists 
		and: [ self arguments size = self method numArgs ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 20:46'!
hash
	^ (((self class hash 
		bitXor: self methodClassName hash)
			bitXor: self methodClass hash)
				bitXor: self meta hash)
					bitXor: self selector hash! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 16:14'!
isValid
	^ self exists! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:59'!
matchesExample: aGTExample
	^ self exists 
		and: [ aGTExample isGTExample
			and: [ aGTExample methodClass = self methodClass
				and: [ aGTExample selector = self selector ] ] ]! !

!GTExampleMethod methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 14:37'!
matchesMethod: aCompiledMethod
	^ self exists
		and: [ self methodClass = aCompiledMethod methodClass
			and: [ self selector = aCompiledMethod selector ] ]! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 20:44'!
meta
	^ meta ifNil: [ meta := false ]! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 20:44'!
meta: anObject
	meta := anObject! !

!GTExampleMethod methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 20:53'!
method
	^ self exists 
		ifTrue: [ self methodClass >> self selector ]
		ifFalse: [ nil ]! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 20:47'!
methodClass
	self ensure.
	^ methodClass! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:04'!
methodClass: anObject
	methodClass := anObject! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:10'!
methodClassName
	^ methodClassName! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 13:10'!
methodClassName: anObject
	methodClassName := anObject! !

!GTExampleMethod methodsFor: 'accessing-dynamic' stamp: 'StefanReichhart 7/3/2016 20:49'!
provider 
	^ self meta
		ifTrue: [ self methodClass ]
		ifFalse: [ self methodClass new ]! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 4/2/2016 17:54'!
selector
	^ selector! !

!GTExampleMethod methodsFor: 'accessing' stamp: 'StefanReichhart 4/2/2016 17:53'!
selector: anObject
	selector := anObject! !

Object subclass: #GTExampleNautilus
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-UI'!

!GTExampleNautilus class methodsFor: 'public' stamp: 'StefanReichhart 7/4/2016 00:03'!
browseAllExamples
	" self browseAllExamples "
	self browseExamples: 'Examples' select: [ :example | true ]! !

!GTExampleNautilus class methodsFor: 'public' stamp: 'StefanReichhart 7/4/2016 08:43'!
browseAllExamplesIgnoredByTests
	" self browseAllExamplesIgnoredByTests "
	self browseExamples: 'Examples ignored by Tests' select: [ :example | 
		example ignoreTest 
			or: [ example ignoreTestReport ] ]! !

!GTExampleNautilus class methodsFor: 'public' stamp: 'StefanReichhart 7/4/2016 08:43'!
browseAllExamplesReferencing: aLiteral
	" self browseAllExamplesReferencing: #and: "
	self browseExamples: 'Examples referencing #', aLiteral asString select: [ :example | 
		example isReferencing: aLiteral ]! !

!GTExampleNautilus class methodsFor: 'public' stamp: 'StefanReichhart 7/4/2016 08:41'!
browseAllExamplesWithDependencies
	" self browseAllExamplesWithDependencies "
	self browseExamples: 'Examples with Dependencies' select: [ :example | 
		example hasArguments ]! !

!GTExampleNautilus class methodsFor: 'public' stamp: 'StefanReichhart 7/4/2016 08:41'!
browseAllExamplesWithProblems
	" self browseAllExamplesWithProblems "
	self browseExamples: 'Examples with Problems' select: [ :example | 
		example hasProblems ]! !

!GTExampleNautilus class methodsFor: 'public' stamp: 'StefanReichhart 7/4/2016 08:41'!
browseAllExamplesWithSubject: aClass
	" self browseAllExamplesWithSubject: GTExample "
	self browseExamples: 'Examples with Subject #', aClass name select: [ :example | 
		example hasSubject: aClass ]! !

!GTExampleNautilus class methodsFor: 'private' stamp: 'StefanReichhart 7/4/2016 08:37'!
browseExamples: aString select: aBlock
	| env |
	env := RBSelectorEnvironment new.
	Smalltalk gtExamplesContained do: [ :example |
		(aBlock value: example) ifTrue: [ 
			env addClass: example methodClass selector: example selector ] ].
	env isEmpty 
		ifTrue: [ ^ UIManager default inform: 'No examples found' ].
	^ env gtBrowseFor: aString! !

!GTExampleNautilus methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 17:27'!
examples: aBlock
	^ aBlock value! !

!GTExampleNautilus methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 22:55'!
examples: aBlock1 collect: aBlock2
	^ aBlock1 value collect: [ :example | 
		example -> (aBlock2 value: example) ]! !

!GTExampleNautilus methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 17:28'!
examples: aBlock1 do: aBlock2
	^ aBlock1 value do: aBlock2! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 7/4/2016 00:30'!
onClass: aBuilder
	| target class |
	
	target := aBuilder model.
	(class := target selectedClass) ifNil: [ ^ self ].
		
	self onExamples: [ (class isMeta 
		ifTrue: [ class ] 
		ifFalse: [ class theNonMetaClass ]) gtExamplesContained ] builder: aBuilder.
	
	(aBuilder item: #'Browse Examples having subject #' ,  class theNonMetaClass name asString)
		parent: #gte;
		order: 90101;
		keyText: 'e, s' if: Nautilus useOldStyleKeys not;
		action: [ self class browseAllExamplesWithSubject: class theNonMetaClass name ]! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 18:41'!
onCompiledMethod: aBuilder
	| target method |
	
	target := aBuilder model.
	(method := target selectedMethod) ifNil: [ ^ self ].

	self onExamples: [ method gtExamples ] builder: aBuilder.
	self onReferences: method selector builder: aBuilder.! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 7/4/2016 08:45'!
onExamples: aBlock builder: aBuilder
	
	(aBuilder item: #gte)
		order: -90000;
		label: 'Examples ...';
		withSeparatorAfter.
	
	(aBuilder item: #'Inspect Examples')
		parent: #gte;
		order: 90000;
		keyText: 'e, i' if: Nautilus useOldStyleKeys not;
		action: [ (self examples: aBlock) inspect ];
		withSeparatorAfter.
	
	(aBuilder item: #'Inspect Return-Values')
		parent: #gte;
		order: 90001;
		keyText: 'e, v' if: Nautilus useOldStyleKeys not;
		action: [ (self examples: aBlock collect: [ :example | example returnValue ]) inspect ].
		
	(aBuilder item: #'Inspect Results (full)')
		parent: #gte;
		order: 90002;
		keyText: 'e, r' if: Nautilus useOldStyleKeys not;
		action: [ (self examples: aBlock collect: [ :example | example result ]) inspect ];
		withSeparatorAfter.
		
	(aBuilder item: #'Run Examples')
		parent: #gte;
		order: 90003;
		keyText: 'e, r' if: Nautilus useOldStyleKeys not;
		action: [ self examples: aBlock do: #run ].
		
	(aBuilder item: 'Debug Examples')
		parent: #gte;
		order: 90004;
		keyText: 'e, d' if: Nautilus useOldStyleKeys not;
		action: [ self examples: aBlock do: #debug ];
		withSeparatorAfter.
		
	(aBuilder item: #'Browse all Examples')
		parent: #gte;
		order: 90005;
		keyText: 'e, a' if: Nautilus useOldStyleKeys not;
		action: [ self class browseAllExamples ].
		
	(aBuilder item: 'Browse all Examples with Problems')
		parent: #gte;
		order: 90006;
		keyText: 'e, p' if: Nautilus useOldStyleKeys not;
		action: [ self class browseAllExamplesWithProblems ].

	(aBuilder item: 'Browse all Examples with Dependencies')
		parent: #gte;
		order: 90007;
		keyText: 'e, c' if: Nautilus useOldStyleKeys not;
		action: [ self class browseAllExamplesWithDependencies ].
		
	(aBuilder item: 'Browse all Examples ignored by Tests')
		parent: #gte;
		order: 90008;
		keyText: 'e, t' if: Nautilus useOldStyleKeys not;
		action: [ self class browseAllExamplesIgnoredByTests ];
		withSeparatorAfter.		! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 17:31'!
onPackage: aBuilder
	| target package |
	
	target := aBuilder model.
	(package := target selectedPackage) ifNil: [ ^ self ].
	package item ifNil: [ ^ self ].
	
	self onExamples: [ package gtExamplesContained ] builder: aBuilder! !

!GTExampleNautilus methodsFor: 'public' stamp: 'StefanReichhart 7/4/2016 08:29'!
onReferences: aLiteral builder: aBuilder
	(aBuilder item: #'Browse Examples referencing #' ,  aLiteral)
		parent: #gte;
		order: 90100;
		keyText: 'e, u' if: Nautilus useOldStyleKeys not;
		action: [ self class browseAllExamplesReferencing: aLiteral ].
		
	(aBuilder item: #'Browse Examples with subject #' ,  aLiteral asString)
		parent: #gte;
		order: 90101;
		keyText: 'e, s' if: Nautilus useOldStyleKeys not;
		action: [ self class browseAllExamplesWithSubject: aLiteral ]! !

Object subclass: #GTExampleOrganizer
	instanceVariableNames: 'theNonMetaClassExamples theMetaClassExamples systemAnnouncements monticelloAnnouncements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

GTExampleOrganizer class
	instanceVariableNames: 'instance'!

GTExampleOrganizer class
	instanceVariableNames: 'instance'!

!GTExampleOrganizer class methodsFor: 'examples' stamp: 'StefanReichhart 9/7/2015 21:41'!
gtExampleInstance
	<gtExample>
	<label: 'An instance of an example organizer'>
	
	^ GTExampleOrganizer instance! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
instance
	^ instance ifNil: [ instance := self basicNew initialize start ]! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 9/7/2015 21:42'!
new
	self shouldNotImplement! !

!GTExampleOrganizer class methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:10'!
registerInterestToSystemAnnouncement
	<systemEventRegistration>
	
	self instance 
		stopSystemAnnouncements;
		stopMonticelloAnnouncements;
		startSystemAnnouncements;
		startMonticelloAnnouncements.! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 20:58'!
reset
	instance ifNotNil: [ instance reset ].
	instance := nil! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:43'!
restart
	self stop.
	self start.! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:43'!
start
	self instance start! !

!GTExampleOrganizer class methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 16:57'!
stop
	instance ifNotNil: [ instance stopThoroughly ].
	self reset! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:44'!
announcer
	^ SystemAnnouncer uniqueInstance weak! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 23:58'!
browserEnvironment
	| environment |
	environment := RBSelectorEnvironment new.
	self examplesDo: [ :className :selector :example |
		environment addClass: example methodClass selector: example selector ].
	^ environment! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/3/2016 19:30'!
categoryRemoved: anAnnouncement
	self withoutExceptionsDo: [ 
		self with: GTExamplesModified announcement: anAnnouncement do: [ :ann |
			self examplesDo: [ :example | ann removeExample: example ].
			self reset; reinitialize.
			self examplesDo: [ :example | ann addExample: example ]. ] ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/3/2016 19:29'!
classRemoved: anAnnouncement
	self withoutExceptionsDo: [ 
		self with: GTExamplesModified announcement: anAnnouncement do: [ :ann |
			ann removeExamples: (theNonMetaClassExamples removeKey: anAnnouncement classRemoved theNonMetaClass name ifAbsent: [ nil ]).
			ann removeExamples: (theMetaClassExamples removeKey: anAnnouncement classRemoved theNonMetaClass name ifAbsent: [ nil ]) ] ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/3/2016 19:36'!
classRenamed: anAnnouncement
	self withoutExceptionsDo: [ 
		self with: GTExamplesModified announcement: anAnnouncement do: [ :ann |
			ann removeExamples: (theNonMetaClassExamples removeKey: anAnnouncement oldName asSymbol ifAbsent: [ nil ]).
			ann removeExamples: (theMetaClassExamples removeKey: anAnnouncement oldName asSymbol ifAbsent: [ nil ]).
			(Smalltalk classNamed: anAnnouncement newName) ifNotNil: [ :class | 
				ann addExamples: class gtExamples ] ] ]! !

!GTExampleOrganizer methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 21:04'!
exampleAt: aCompiledMethod ifAbsentPut: aBlock
	^ self exampleAtClass: aCompiledMethod methodClass selector: aCompiledMethod selector ifAbsentPut: aBlock! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 23:56'!
exampleAtClass: class selector: selector ifAbsentPut: aBlock
	^ ((class isMeta 
		ifTrue: [ theMetaClassExamples ]
		ifFalse: [ theNonMetaClassExamples ]) 
			at: class name
			ifAbsentPut: [ WeakIdentityKeyDictionary new: 100 ]) 
				at: selector 
				ifAbsentPut: aBlock! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 19:33'!
examplesAnnouncer
	^ GTExampleAnnouncer instance! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/3/2016 17:00'!
examplesCollect: aBlock
	| examples |
	examples := OrderedCollection new.
	self examplesDo: [ :class :method :example | 
		examples add: (aBlock value: example) ].
	^ examples! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/3/2016 17:01'!
examplesDetect: aBlock
	self examplesDo: [ :class :selector :example | 
		(aBlock value: example) ifTrue: [ ^ example ] ].
	^ nil! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/3/2016 17:00'!
examplesDo: aBlock
	theNonMetaClassExamples keysAndValuesDo: [ :className :methodDict |
		methodDict keysAndValuesDo: [ :selector :example | 
			aBlock value: className value: selector value: example ] ].
	theMetaClassExamples keysAndValuesDo: [ :className :methodDict |
		methodDict keysAndValuesDo: [ :selector :example | 
			aBlock value: className value: selector value: example ] ]! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/3/2016 17:02'!
examplesReject: aBlock
	^ self examplesSelect: [ :example | (aBlock value: example) not ]! !

!GTExampleOrganizer methodsFor: 'enumeration' stamp: 'StefanReichhart 7/3/2016 17:01'!
examplesSelect: aBlock
	| examples |
	examples := OrderedCollection new.
	self examplesDo: [ :class :selector :example | 
		(aBlock value: example) ifTrue: [ examples add: example ] ].
	^ examples! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 23:46'!
gtExamplesContained
	^ self examplesCollect: [ :each | each ]! !

!GTExampleOrganizer methodsFor: 'ui' stamp: 'StefanReichhart 7/3/2016 15:46'!
gtSpotterGTExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ Smalltalk gtExamplesContained ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!GTExampleOrganizer methodsFor: 'initializing' stamp: 'StefanReichhart 7/3/2016 23:54'!
initialize
	super initialize.
	
	theNonMetaClassExamples := WeakIdentityKeyDictionary new: 10000.
	theMetaClassExamples := WeakIdentityKeyDictionary new: 10000.! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/3/2016 17:03'!
loadStarted: anAnnouncement
	self withoutExceptionsDo: [
		self stopSystemAnnouncements ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 11/29/2015 15:00'!
loadStopped: anAnnouncement
	self withoutExceptionsDo: [ 
		self startSystemAnnouncements ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/3/2016 19:28'!
methodAdded: anAnnouncement
	self withoutExceptionsDo: [ 
		self with: GTExamplesModified announcement: anAnnouncement do: [ :ann |
			ann addExample: anAnnouncement method gtExample ].
		GTExampleNautilusAction methodChanged: anAnnouncement method. ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/3/2016 19:28'!
methodModified: anAnnouncement
	self withoutExceptionsDo: [ 
		self with: GTExamplesModified announcement: anAnnouncement do: [ :ann |
			theNonMetaClassExamples removeKey: anAnnouncement classRemoved theNonMetaClass name thenDo: [ :methodDict |
				ann removeExample: (methodDict removeKey: anAnnouncement method selector ifAbsent: [ nil ]) ].
			theMetaClassExamples removeKey: anAnnouncement classRemoved theNonMetaClass name thenDo: [ :methodDict |
				ann removeExample: (methodDict removeKey: anAnnouncement method selector ifAbsent: [ nil ]) ].
			ann addExample: anAnnouncement method gtExample ].
		GTExampleNautilusAction methodChanged: anAnnouncement method. ]! !

!GTExampleOrganizer methodsFor: 'private-events' stamp: 'StefanReichhart 7/3/2016 19:28'!
methodRemoved: anAnnouncement
	self withoutExceptionsDo: [ 
		self with: GTExamplesModified announcement: anAnnouncement do: [ :ann |
			theNonMetaClassExamples removeKey: anAnnouncement oldMethod methodClass theNonMetaClass name thenDo: [ :methodDict |
				ann removeExample: (methodDict removeKey: anAnnouncement oldMethod selector ifAbsent: [ nil ]) ].
			theMetaClassExamples removeKey: anAnnouncement oldMethod methodClass theNonMetaClass name thenDo: [ :methodDict |
				ann removeExample: (methodDict removeKey: anAnnouncement oldMethod selector ifAbsent: [ nil ]) ] ].
		GTExampleNautilusAction methodChanged: anAnnouncement method. ]! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 16:55'!
reinitialize
	SmalltalkImage current gtExamplesContained! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:02'!
reset
	self initialize! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 16:55'!
restart
	self stopThoroughly.
	self start.! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
start
	self startMonticelloAnnouncements.
	self startSystemAnnouncements! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:11'!
startMonticelloAnnouncements
	monticelloAnnouncements == true ifTrue: [ ^ self ].
	self announcer
		when: MCVersionLoaderStarted send: #loadStarted: to: self;
		when: MCVersionLoaderStopped send: #loadStopped: to: self.
	monticelloAnnouncements := true! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:11'!
startSystemAnnouncements
	systemAnnouncements == true ifTrue: [ ^ self ].
	self announcer
		when: MethodAdded send: #methodAdded: to: self;
		when: MethodModified send: #methodModified: to: self;
		when: MethodRemoved send: #methodRemoved: to: self;
		when: ClassRemoved send: #classRemoved: to: self;
		when: ClassRenamed send: #classRenamed: to: self;
		when: CategoryRemoved send: #categoryRemoved: to: self.
	systemAnnouncements := true! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
stop
	self stopMonticelloAnnouncements.
	self stopSystemAnnouncements! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 22:18'!
stopMonticelloAnnouncements
	self announcer 
		unsubscribe: MCVersionLoaderStarted thoroughlyFor: self;
		unsubscribe: MCVersionLoaderStopped thoroughlyFor: self.
	monticelloAnnouncements := false! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 22:18'!
stopSystemAnnouncements
	self announcer 
		unsubscribe: MethodAdded thoroughlyFor: self;
		unsubscribe: MethodModified thoroughlyFor: self;
		unsubscribe: MethodRemoved thoroughlyFor: self;
		unsubscribe: ClassRemoved thoroughlyFor: self;
		unsubscribe: ClassRenamed thoroughlyFor: self;
		unsubscribe: CategoryRemoved thoroughlyFor: self.
	systemAnnouncements := false! !

!GTExampleOrganizer methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:12'!
stopThoroughly
	self stop.
	
	Announcer allSubInstances do: [ :each | each unsubscribeThoroughly: self ]! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 19:34'!
with: aClass announcement: announcement do: aBlock
	| wrappedAnnouncement |
	wrappedAnnouncement := aClass new
		announcement: announcement;
		yourself.
	aBlock value: wrappedAnnouncement.
	self examplesAnnouncer announce: wrappedAnnouncement! !

!GTExampleOrganizer methodsFor: 'private' stamp: 'StefanReichhart 7/2/2016 21:14'!
withoutExceptionsDo: aBlock
	[ aBlock value ]
		on: Error
		do: [ :exception | " we should not ignore it, but try to continue " ]! !

Object subclass: #GTExamplePainter
	instanceVariableNames: 'example elements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-TODO'!

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 14:42'!
borderColorForExample: anExample
	^ [ self colorForResult: anExample result ]
		on: Error
		do: [ self colorForExample: anExample ]! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 17:09'!
buildEdges: aBlock inView: aView
	| edges container fromElement toElements elementsWithModels |
	elements isEmpty ifTrue: [ ^ #() ].
	edges := OrderedCollection new.
	elementsWithModels := elements select: [ :el | el model notNil ].
	
	elementsWithModels do: [ :element |
		container := element view.
		fromElement := container elementFromModel: (#yourself rtValue: element model).
		
		"We get the list of elements that corresponds to (toBlock roValues: element model)"
		toElements := OrderedCollection new.
		([ :node | self dependenciesOrArgumentsForExample: node ] rtValue: element model) do: [ :model | 
			| v |
			v := container elementFromModel: model.
			v notNil ifTrue: [ toElements add: v ] ].
		
		
		(fromElement notNil and: [ toElements notEmpty ])
			ifTrue: [  
				toElements do: [ :toElement |
					| edge line |
					edge := RTEdge from: fromElement to: toElement. 
					line := RTArrowedLine new 
						color: Color black; 
						attachPoint: RTVerticalAttachPoint instance;
						yourself.
					aBlock value: edge value: fromElement value: toElement value: line.
					edge + line.
					edges add: edge.
					container add: edge ] ]  ].
	aView addAll: edges.! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 17:29'!
buildElements: aRootExample inView: aView
	| traverser |
	" init "
	elements := RTGroup new.
	
	" root element "
	elements add: (self elementForExample: aRootExample with: [ :shape | shape size: 48 ]).
	
	" top-down "
	(traverser := aRootExample traverser)
		onRecursionError: [ :aProcessor :anExample | self onRecursionError: anExample ];
		onArgumentError: [ :aProcessor :anExample :someArguments | self onArgumentError: anExample ];
		onDependencyError: [ :aProcessor :anExample :anException | self onDependencyError: anExample ];
		action: [ :anExample | self onAction: anExample ];
		value.
	
	" bottom up "
	traverser reversed value.
	
	" add to view "
	aView addAll: elements @ (RTPopup text: [ :node | node gtDisplayString ]) @ RTDraggable.! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 15:18'!
buildLayoutinView: aView
	" layout -> graph instead of tree "
	RTTreeLayout new on: elements! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 14:58'!
colorForExample: anExample
	^ anExample exists 
		ifTrue: [ anExample isValid 
			ifTrue: [ self defaultColorForExample: anExample ] 
			ifFalse: [ Color red ] ] 
		ifFalse: [ (Color r: 1 g: 0 b: 0 alpha: 0.05) ]! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 14:58'!
colorForResult: aResult
	aResult isSuccess ifTrue: [ ^ Color green ].
	aResult isFailure ifTrue: [ ^ Color yellow ].
	aResult isError ifTrue: [ ^ Color red ].
	^ self defaultColorForExample: nil! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 14:57'!
defaultColorForExample: anExample
	^ Color black! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 8/31/2015 22:52'!
dependenciesOrArgumentsForExample: anExample
	^ [ anExample arguments collect: [ :each | 
		[ each example ] 
			on: Error 
			do: [ :exception | each ] ] ] on: Error do: [ #() ]! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 8/31/2015 22:28'!
elementByExample: anExample
	^ elements 
		detect: [ :element | element model == anExample ] 
		ifNone: [ nil ]! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 17:04'!
elementByExample: anExample do: aBlock1 ifNone: aBlock2
	(self elementByExample: anExample)
		ifNotNil: aBlock1
		ifNil: aBlock2! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 8/31/2015 22:38'!
elementForExample: anExample
	^ self elementForExample: anExample with: [ :shape | ]! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 14:57'!
elementForExample: anExample with: aBlock
	| shape |
	shape := RTBox new 
		size: 24;
		color: (self defaultColorForExample: anExample);
		borderColor: (self borderColorForExample: anExample);
		borderWidth: 2;
		yourself.
	aBlock value: shape.
	^ shape elementOn: anExample! !

!GTExamplePainter methodsFor: 'accessing' stamp: 'StefanReichhart 8/31/2015 22:26'!
example
	^ example! !

!GTExamplePainter methodsFor: 'accessing' stamp: 'StefanReichhart 8/31/2015 22:26'!
example: anObject
	example := anObject! !

!GTExamplePainter methodsFor: 'public' stamp: 'StefanReichhart 8/31/2015 22:56'!
gtInspectorRoassalIn: composite
	self hasRoassal2 ifFalse: [ ^ self ].
	^ composite roassal2
		title: 'Graph';
		painting: [ :view :node | self viewGraph: node on: view ];
		yourself! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 8/31/2015 22:57'!
hasRoassal2
	^ (Smalltalk classNamed: #GLMRoassal2Presentation) notNil! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/2/2015 21:20'!
onAction: anExample
	self elementByExample: anExample
		do: [ :element | ]
		ifNone: [ elements add: (self elementForExample: anExample) ] ! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 14:43'!
onArgumentError: anExample
	self elementByExample: anExample
		do: [ :element | 
			element color: Color red.
			(self dependenciesOrArgumentsForExample: anExample) do: [ :argument |
				self elementByExample: argument
					do: [ :subElement | ]
					ifNone: [ elements add: (self elementForExample: argument with: [ :shape | shape 
							color: (self colorForExample: argument); 
							borderColor: (self borderColorForExample: argument) ]) ] ] ]
		ifNone: [ ] ! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 14:38'!
onDependencyError: anExample
	self elementByExample: anExample
		do: [ :element | 
			anExample hasValidArguments 
				ifFalse: [ element color: Color red ] ]
		ifNone: [ 
			elements add: (self elementForExample: anExample) ].
		
	anExample arguments do: [ :argument |
		elements add: (self elementForExample: argument with: [ :shape | shape 
			color: (self colorForExample: argument); 
			borderColor: (self borderColorForExample: argument) ]) ] ! !

!GTExamplePainter methodsFor: 'private' stamp: 'StefanReichhart 9/2/2015 21:19'!
onRecursionError: anExample
	self elementByExample: anExample
		do: [ :element | element color: Color yellow ]
		ifNone: [  ] ! !

!GTExamplePainter methodsFor: 'private-painting' stamp: 'StefanReichhart 9/4/2015 15:23'!
viewGraph: aRootExample on: aView
	self buildElements: aRootExample inView: aView.
	self buildEdges: [ :edge :from :to :line | 
		from model hasValidArguments ifFalse: [ line color: Color red ] ] inView: aView.
	self buildLayoutinView: aView.
	
	^ aView! !

Object subclass: #GTExampleProcessor
	instanceVariableNames: 'example context processing onProcess onRecursionError onDependencyError onArgumentError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

GTExampleProcessor subclass: #GTExampleEvaluator
	instanceVariableNames: 'processed result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

GTExampleEvaluator subclass: #GTExampleDebugger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleDebugger methodsFor: 'private' stamp: 'StefanReichhart 7/13/2015 12:14'!
do: aBlock on: anException do: anotherBlock
	aBlock value! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 7/13/2015 12:14'!
do: aBlock on: anException do: anotherBlock
	aBlock
		on: anException
		do: anotherBlock! !

!GTExampleEvaluator methodsFor: 'initializing' stamp: 'StefanReichhart 5/13/2016 11:04'!
initialize
	super initialize.
	
	processed := OrderedCollection new! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 5/13/2016 11:38'!
newResultFor: anExample
	^ GTExampleResult new 
		example: anExample;
		context: self context;
		yourself.! !

!GTExampleEvaluator methodsFor: 'primitives' stamp: 'StefanReichhart 5/13/2016 10:31'!
primitiveProcessExample: anExample withArguments: arguments
	^ anExample provider perform: anExample method selector withArguments: arguments! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 20:24'!
process: anExample withArguments: aBlock
	| currentResult arguments |
	currentResult := self newResultFor: anExample.
	arguments := nil.
	anExample hasValidArguments 
		ifFalse: [ ^ self onArgumentError value: self value: anExample value: arguments ].
	[ arguments := aBlock value ]
		on: Error
		do: [ :exception | 
			currentResult unexpectedError: exception.
			exception class = GTExampleRecursionDetected ifTrue: [ exception pass ].
			^ self onDependencyError value: self value: anExample value: exception ].
	self do: [ currentResult returnValue: (self primitiveProcessExample: anExample withArguments: arguments) ]
		on: anExample signalableExceptions
		do: [ :exception | currentResult expectedError: exception ].
	processed add: currentResult.
	self do: [ self processAfter: anExample ]
		on: anExample signalableExceptions
		do: [ :exception | currentResult expectedError: exception ].
	anExample == self example ifTrue: [ result := currentResult ].
	anExample exceptions signaledBy: currentResult expectedError.
	^ currentResult returnValue! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 6/24/2016 11:52'!
processAfter: anExample
	anExample == self example ifFalse: [ ^ self ].
	processed reverseDo: [ :processedResult |
		self processAfter: processedResult example withResult: processedResult returnValue ]! !

!GTExampleEvaluator methodsFor: 'private' stamp: 'StefanReichhart 6/24/2016 12:17'!
processAfter: anExample withResult: anObject
	anExample after ifNil: [ ^ self ].
	anExample after method
		valueWithReceiver: anExample after provider 
		possibleArguments: { anObject }! !

!GTExampleEvaluator methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 20:25'!
result
	result := self newResultFor: self example.
	self do: [ self value ] 
		on: Error
		do: [ :exception | result returnValue: nil; unexpectedError: exception ].
	GTExampleNautilusAction methodUpdated: self example method with: result.
	^ result! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 19:47'!
basicProcess: anExample 
	^ self 
		process: anExample 
		withArguments: [ anExample arguments collect: [ :each | self process: each gtExample ] ] ! !

!GTExampleProcessor methodsFor: 'testing' stamp: 'StefanReichhart 9/6/2015 17:44'!
canProcess
	[ self value ]
		on: GTExampleError
		do: [ ^ false ].
	^ true! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:49'!
context
	^ context! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:49'!
context: anObject
	^ context := anObject! !

!GTExampleProcessor methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 8/31/2015 22:06'!
defaultOnArgumentError
	^ [ :aProcessor :anExample :someArguments | 
		aProcessor handleArgumentErrorFor: anExample withArguments: someArguments.
		nil ]! !

!GTExampleProcessor methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 8/31/2015 22:06'!
defaultOnDependencyError
	^ [ :aProcessor :anExample :anException | 
		aProcessor handleDependencyErrorFor: anExample withException: anException.
		nil ]! !

!GTExampleProcessor methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 9/2/2015 20:58'!
defaultOnProcess
	^ [ :aProcessor :anExample | 
		aProcessor basicProcess: anExample ]! !

!GTExampleProcessor methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 8/31/2015 22:06'!
defaultOnRecursionError
	^ [ :aProcessor :anExample | 
		aProcessor handleRecursionErrorFor: anExample.
		nil ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 14:48'!
example
	^ example! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 14:48'!
example: anObject
	example := anObject! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 17:44'!
handleArgumentErrorFor: anExample withArguments: arguments
	^ (GTExampleArgumentError new
		example: anExample;
		expected: anExample method argumentNames;
		defined: anExample arguments;
		evaluated: arguments;
		yourself) signal! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 9/6/2015 17:43'!
handleDependencyErrorFor: anExample withException: anException
	^ (GTExampleDependencyError new
		example: anExample;
		exception: anException;
		yourself) signal! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 20:15'!
handleRecursionErrorFor: anExample
	^ (GTExampleRecursionDetected new 
		example: self example;
		resolved: processing;
		next: anExample;
		yourself) signal! !

!GTExampleProcessor methodsFor: 'initializing' stamp: 'StefanReichhart 3/28/2016 20:17'!
initialize
	super initialize.
	
	processing := IdentitySet new.
	context := GTExampleContext new! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 8/29/2015 19:17'!
onArgumentError
	^ onArgumentError ifNil: [ onArgumentError := self defaultOnArgumentError ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 8/29/2015 19:17'!
onArgumentError: anObject
	onArgumentError := anObject! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/27/2015 14:29'!
onDependencyError
	^ onDependencyError ifNil: [ onDependencyError := self defaultOnDependencyError ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/27/2015 14:29'!
onDependencyError: anObject
	onDependencyError := anObject! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 15:21'!
onProcess
	^ onProcess ifNil: [ onProcess := self defaultOnProcess ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 15:21'!
onProcess: anObject
	onProcess := anObject! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 16:02'!
onRecursionError
	^ onRecursionError ifNil: [ onRecursionError := self defaultOnRecursionError ]! !

!GTExampleProcessor methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 15:57'!
onRecursionError: anObject
	onRecursionError := anObject! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 8/23/2015 15:23'!
process: anExample
	^ self process: anExample withoutLoopsDo: [ 
		self onProcess value: self value: anExample ]! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 9/4/2015 16:55'!
process: anExample withArguments: aBlock
	self subclassResponsibility! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 22:00'!
process: anExample withoutLoopsDo: aBlock
	| value |
	" check whether we already scheduled the given example for processing -> prevent recursion "
	value := nil.
	processing do: [ :processingExample |
		(processingExample = anExample) ifTrue: [
			^ self onRecursionError value: self value: anExample ] ].	
	processing add: anExample. " prepare processing this example "
	value := aBlock value. " process this example "
	processing remove: anExample. " done processing "
	^ value! !

!GTExampleProcessor methodsFor: 'public' stamp: 'StefanReichhart 3/28/2016 18:55'!
value 
	^ self withContextDo: [ self process: self example ]! !

!GTExampleProcessor methodsFor: 'private' stamp: 'StefanReichhart 3/28/2016 20:17'!
withContextDo: aBlock
	GTCurrentExampleContext use: context during: aBlock! !

GTExampleProcessor subclass: #GTExampleTraverser
	instanceVariableNames: 'action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleTraverser methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 15:04'!
action
	^ action! !

!GTExampleTraverser methodsFor: 'accessing' stamp: 'StefanReichhart 7/26/2015 15:04'!
action: anObject
	action := anObject! !

!GTExampleTraverser methodsFor: 'private' stamp: 'StefanReichhart 8/31/2015 22:11'!
process: anExample withArguments: aBlock
	| arguments |
	arguments := nil.
	anExample hasValidArguments 
		ifFalse: [ ^ self onArgumentError value: self value: anExample value: arguments ].
	[ arguments := aBlock value ]
		on: Error
		do: [ :exception | ^ self onDependencyError value: self value: anExample value: exception ].
	self action ifNotNil: [ self example == anExample ifFalse: [ self action value: anExample ] ]! !

!GTExampleTraverser methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 16:46'!
reversed
	self onProcess: [ :processor :anExample | 
		self action ifNotNil: [ self example == anExample ifFalse: [ self action value: anExample ] ].
		anExample directDependents do: [ :each | processor process: each ] ]! !

Object subclass: #GTExampleResult
	instanceVariableNames: 'example context returnValue expectedError unexpectedError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:54'!
context
	^ context! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 3/28/2016 18:54'!
context: anObject
	context := anObject! !

!GTExampleResult methodsFor: 'actions' stamp: 'StefanReichhart 8/25/2015 21:36'!
debug
	self isSuccess ifTrue: [ ^ self ].
	^ self example debug! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 6/28/2015 16:53'!
example
	^ example! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 6/28/2015 16:53'!
example: anObject
	example := anObject! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:46'!
expectedError
	^ expectedError! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:46'!
expectedError: anObject
	expectedError := anObject! !

!GTExampleResult methodsFor: 'ui' stamp: 'StefanReichhart 4/10/2016 11:09'!
gtDisplayOn: aStream
	self example gtDisplayOn: aStream. 
	aStream nextPutAll: ' --> '.
	self unexpectedError 
		ifNotNil: [ self unexpectedError gtDisplayOn: aStream ]
		ifNil: [ self returnValue gtDisplayOn: aStream ].
! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 7/3/2016 22:54'!
hasProblems
	^ self example hasProblems
		or: [ self isFailure
			or: [ self isError ] ]! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:47'!
isError
	^ self isSuccess not
		and: [ self unexpectedError isExampleError ]! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:53'!
isExpectedError
	^ self expectedError notNil! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:47'!
isFailure
	^ self isSuccess not
		and: [ self unexpectedError isExampleError not ]! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:47'!
isSuccess
	^ self unexpectedError isNil! !

!GTExampleResult methodsFor: 'testing' stamp: 'StefanReichhart 8/23/2015 13:54'!
isUnexpectedError
	^ self unexpectedError notNil! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/2/2015 11:59'!
returnValue
	^ returnValue! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/2/2015 11:59'!
returnValue: anObject
	returnValue := anObject! !

!GTExampleResult methodsFor: 'actions' stamp: 'StefanReichhart 8/25/2015 21:36'!
run
	^ self example run! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:46'!
unexpectedError
	^ unexpectedError! !

!GTExampleResult methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:47'!
unexpectedError: anObject
	unexpectedError := anObject! !

!Object class methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 3/28/2016 18:53'!
gtExampleContext
	^ GTCurrentExampleContext value! !

!Object class methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:35'!
gtExamplesSubjects
	^ {  }! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:42'!
asGTExampleSubject
	^ GTExampleInstanceSubject new 
		instance: self;
		yourself! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 3/28/2016 18:52'!
gtExampleContext
	^ self class gtExampleContext! !

!Object methodsFor: '*GT-Examples-UI' stamp: 'TudorGirba 1/27/2016 00:18'!
gtExampleSourceIn: composite inContext: aGTInspector
	<gtInspectorPresentationOrder: 100>
	| previousRawSelection |
	"we first check to see if we have at least two panes, because only
	if we have two panes does it make sense to check the raw selection of the previous pane"
	((aGTInspector isKindOf: GTInspector) and: [aGTInspector first panes size >= 2]) ifFalse: [ ^ self ].
	
	previousRawSelection := (aGTInspector first panes reverse second port: #rawSelection) value.
	previousRawSelection isGTExample ifTrue: [ 
		previousRawSelection gtInspectorSourceIn: composite ]! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/6/2015 16:32'!
gtExamples
	^ self class gtExamples! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:31'!
gtExamplesContained
	^ self gtExamples! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:50'!
gtExamplesFactory
	^ self class gtExamplesFactory! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:36'!
gtExamplesSubjects
	^ {  }! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 21:58'!
isGTExample
	^ false! !

!Object methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 13:33'!
isGTExampleMethod
	^ false! !

!WeakSubscriptionBuilder methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 22:18'!
unsubscribe: anAnnouncementClass thoroughlyFor: aSubscriber
	announcer unsubscribe: anAnnouncementClass thoroughlyFor: aSubscriber! !

!SmalltalkImage methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 22:20'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	Object withAllSubclassesDo: [ :class |
		class gtExamplesFactory examplesDo: [ :example | 
			examples add: example ] ].
	^ examples! !

!SmalltalkImage methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 15:47'!
gtSpotterGTExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ self gtExamplesContained ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

Notification subclass: #GTCurrentExampleContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTCurrentExampleContext class methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/28/2016 18:45'!
defaultValue
	^ nil! !

!GTCurrentExampleContext class methodsFor: 'evaluating' stamp: 'StefanReichhart 3/28/2016 18:45'!
use: anObject during: aBlock
	^ aBlock 
		on: self 
		do: [ :notification | notification resume: anObject ]! !

!GTCurrentExampleContext class methodsFor: 'evaluating' stamp: 'StefanReichhart 3/28/2016 18:45'!
value
	^ self signal! !

!GTCurrentExampleContext methodsFor: 'accessing-defaults' stamp: 'StefanReichhart 3/28/2016 18:44'!
defaultAction
	^ self class defaultValue! !

!RPackage methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:49'!
gtExamplesContained
	| examples |
	examples := OrderedCollection new.
	self methods do: [ :method |
		method gtExamplesFactory exampleMethod: method do: [ :example | 
			examples add: example ] ].
	^ examples! !

IdentityDictionary subclass: #GTExampleContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!Announcer methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 22:18'!
unsubscribe: anAnnouncementClass thoroughlyFor: aSubscriber
	^ registry protected: [
		registry subscriptions removeAllSuchThat: [ :subscription | 
			subscription subscriber class == aSubscriber class " class !!!!!! "
			 and: [ subscription announcementClass == anAnnouncementClass ] ] ]
! !

!Announcer methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 2/26/2016 10:41'!
unsubscribeThoroughly: aSubscriber
	^ registry protected: [
		registry subscriptions removeAllSuchThat: [ :subscription | 
			subscription subscriber class == aSubscriber class " class !!!!!! " ] ]
! !

Announcer subclass: #GTExampleAnnouncer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

GTExampleAnnouncer class
	instanceVariableNames: 'instance'!

GTExampleAnnouncer class
	instanceVariableNames: 'instance'!

!GTExampleAnnouncer class methodsFor: 'public' stamp: 'StefanReichhart 7/2/2016 21:14'!
instance
	^ instance ifNil: [ instance := self basicNew initialize ]! !

!SubscriptionRegistry methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 11/29/2015 15:27'!
subscriptions
	^ subscriptions! !

TestCase subclass: #GTExampleTestCase
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Testing'!

!GTExampleTestCase class methodsFor: 'testing' stamp: 'StefanReichhart 9/26/2015 11:24'!
isAbstract
	^ self name = #GTExampleTestCase! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 22:12'!
package: package matchesPackagePattern: packageName
	^ package packageName = packageName
		or: [ (packageName match: package packageName) 
			or: [ [ package packageName matchesRegexIgnoringCase: packageName ] 
						on: Error 
						do: [ false ] ] ]! !

!GTExampleTestCase methodsFor: 'configuration' stamp: 'StefanReichhart 9/26/2015 10:17'!
packageNames
	^ {  }! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 22:11'!
packages: aCollection
	| packages |
	packages := OrderedCollection new.
	RPackageOrganizer default packages do: [ :package |
		aCollection do: [ :packageName |
			(self package: package matchesPackagePattern: packageName) ifTrue: [ 
				packages add: package ] ] ].
	^ packages! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 9/26/2015 11:11'!
run: aResult
	result := aResult.
	[ super run: aResult ] ensure: [ result := nil ]! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 9/26/2015 11:12'!
runCase
	result ifNil: [ result := self classForTestResult new " fake for debug runner " ].
	super runCase. ! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 6/21/2016 15:56'!
runExample: example
	" this is not the testing tool we have in mind, but a temporary solution to integrate examples into the currently existing testing tools "
	| exampleResult |
	example ignoreTest ifTrue: [ ^ self ].
	self shouldnt: [ exampleResult := example run ] raise: Error.
	exampleResult isError 
		ifTrue: [ ^ self classForTestResult signalErrorWith: exampleResult gtDisplayString  ].
	exampleResult isFailure 
		ifTrue: [ ^ self classForTestResult signalFailureWith: exampleResult gtDisplayString  ] ! !

!GTExampleTestCase methodsFor: 'private' stamp: 'StefanReichhart 6/21/2016 15:58'!
runExampleContainers: aCollection
	| allExamples |
	(allExamples := aCollection gather: [ :container | container gtExamplesContained ])
		do: [ :example | self runExample: example ]
		displayingProgress: 'Running ' , allExamples size asString , ' Examples'! !

!GTExampleTestCase methodsFor: 'tests' stamp: 'StefanReichhart 9/26/2015 11:17'!
testExamples
	" default testcase "
	self runExampleContainers: (self packages: self packageNames)! !

RBBlockLintRule subclass: #GTExampleNotDefinedRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Testing'!

!GTExampleNotDefinedRule commentStamp: 'AndreiChis 1/22/2016 16:20' prior: 0!
Every class that defines a custom extension for GTSpotter or GTInspector should define also, at least, a GTExample.!

!GTExampleNotDefinedRule class methodsFor: 'testing' stamp: 'AndreiChis 1/22/2016 15:46'!
checksClass
	^ true! !

!GTExampleNotDefinedRule class methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 16:02'!
uniqueIdentifierName
	"This number should be unique and should change only when the rule completely change semantics"
	
	^'GTExampleNotDefinedRule'! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 15:56'!
basicCheck: aClass 
	^ (self containsCustomExtension: aClass) 
		and: [ (self containsGTExample: aClass) not ]! !

!GTExampleNotDefinedRule methodsFor: 'as yet unclassified' stamp: 'AndreiChis 1/22/2016 16:15'!
check: aClass forCriticsDo: aCriticBlock ifNone: alternativeBlock
	(self basicCheck: aClass)
		ifTrue:  [ aCriticBlock cull: (
			(ReMissingMethodCritic
				for: aClass
				by: self
				class: aClass class
				selector: #gtExample) beShouldBeImplemented )  ]
		ifFalse: alternativeBlock ! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 16:10'!
checkClass: aClass 
	(self basicCheck: aClass) ifTrue: [
		result
			addClass: aClass ]! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 16:09'!
containsCustomExtension: aClass

	^ aClass methods anySatisfy: [:aMethod | 
		aMethod pragmas anySatisfy: [ :aPragma |
			aPragma keyword = GTInspector extensionsPragma or: [
				aPragma keyword = GTSpotter extensionsPragma ]]]! !

!GTExampleNotDefinedRule methodsFor: 'running' stamp: 'AndreiChis 1/22/2016 15:59'!
containsGTExample: aClass

	^ aClass class methods anySatisfy: [:aMethod | 
			aMethod pragmas 
				anySatisfy: [ :pragma | pragma isGTExamplePragma ]]! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 15:46'!
group
	^ 'Design Flaws'! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 16:00'!
name
	^ 'Class defines a custom GT extension but no example'! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 15:46'!
resultClass
	^ RBClassEnvironment! !

!GTExampleNotDefinedRule methodsFor: 'accessing' stamp: 'AndreiChis 1/22/2016 15:46'!
severity
	^ #information! !

!Exception methodsFor: '*GT-Examples-Exceptions' stamp: 'StefanReichhart 7/13/2015 11:27'!
isExampleError
	^ false! !

Announcement subclass: #GTExamplesModified
	instanceVariableNames: 'announcement added removed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Utils'!

!GTExamplesModified methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 19:25'!
addExample: anExample
	anExample ifNil: [ ^ self ].
	added add: anExample! !

!GTExamplesModified methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 19:37'!
addExamples: aCollection
	aCollection ifNil: [ ^ self ].
	aCollection do: [ :value | self addExample: value ]! !

!GTExamplesModified methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 19:25'!
announcement
	^ announcement! !

!GTExamplesModified methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 19:25'!
announcement: anObject
	announcement := anObject! !

!GTExamplesModified methodsFor: 'initializing' stamp: 'StefanReichhart 7/3/2016 19:31'!
initialize
	super initialize.
	
	added := OrderedCollection new: 100.
	removed := OrderedCollection new: 100.! !

!GTExamplesModified methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 19:25'!
removeExample: anExample
	anExample ifNil: [ ^ self ].
	removed add: anExample! !

!GTExamplesModified methodsFor: 'public' stamp: 'StefanReichhart 7/3/2016 19:37'!
removeExamples: aCollection
	aCollection ifNil: [ ^ self ].
	aCollection do: [ :value | self removeExample: value ]! !

!GTSpotter methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/3/2016 15:49'!
gtSpotterExamplesFor: aStep
	<spotterOrder: 41>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ Smalltalk gtExamplesContained ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring;
		wantsToDisplayOnEmptyQuery: false! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:43'!
asGTExampleSubject
	^ GTExampleMethodSubject new 
		subjectClass: self methodClass;
		subjectSelector: self selector;
		yourself! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 13:37'!
gtExample
	self gtExamplesFactory exampleMethod: self do: [ :example | ^ example ].
	^ nil! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 5/29/2016 12:22'!
gtExampleValue
	^ self gtExample returnValue! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 16:17'!
gtExamples
	^ { self gtExample }! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 13:32'!
isGTExampleMethod
	^ self pragmas anySatisfy: [ :each | each isGTExamplePragma ]! !

!CompiledMethod methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 6/24/2016 12:17'!
valueWithReceiver: aReceiver possibleArguments: anArray 
	^ anArray size = self numArgs 
		ifTrue: [ self valueWithReceiver: aReceiver arguments: anArray  ]
		ifFalse: [ self valueWithReceiver: aReceiver arguments: (anArray copyFrom: 1 to: self numArgs) ]! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/3/2016 22:02'!
asGTExampleSubject
	^ GTExampleClassSubject new 
		subjectClass: self;
		yourself! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:13'!
gtExamples
	^ self gtExamplesFactory gtExamplesContained! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 18:58'!
gtExamplesFactory
	^ GTExampleFactory new
		source: self;
		yourself! !

!TClassDescription methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 20:25'!
gtExamplesSubjects
	^ {  }! !

!Error methodsFor: '*GT-Examples-Exceptions' stamp: 'StefanReichhart 7/3/2016 12:22'!
asGTExamplePragmaError
	^ GTExamplePragmaError new 
		signaledException: self;
		yourself! !

Error subclass: #GTExampleError
	instanceVariableNames: 'example signaledException'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

GTExampleError subclass: #GTExampleArgumentError
	instanceVariableNames: 'expected defined evaluated'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:29'!
defined
	^ defined! !

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:29'!
defined: anObject
	defined := anObject! !

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:28'!
evaluated
	^ evaluated! !

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:28'!
evaluated: anObject
	evaluated := anObject! !

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:17'!
expected
	^ expected! !

!GTExampleArgumentError methodsFor: 'accessing' stamp: 'StefanReichhart 7/24/2015 18:17'!
expected: anObject
	expected := anObject! !

GTExampleError subclass: #GTExampleDependencyError
	instanceVariableNames: 'exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleDependencyError methodsFor: 'accessing' stamp: 'StefanReichhart 9/2/2015 22:02'!
exception
	^ exception! !

!GTExampleDependencyError methodsFor: 'accessing' stamp: 'StefanReichhart 9/2/2015 22:02'!
exception: anObject
	exception := anObject! !

!GTExampleDependencyError methodsFor: 'testing' stamp: 'StefanReichhart 9/4/2015 15:40'!
isExampleError
	^ self exception 
		ifNotNil: [ self exception isExampleError ]
		ifNil: [ super isExampleError ]! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:01'!
example
	^ example! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:01'!
example: anObject
	example := anObject! !

!GTExampleError methodsFor: 'testing' stamp: 'StefanReichhart 7/13/2015 12:03'!
isExampleError
	^ true! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:18'!
signaledException
	^ signaledException! !

!GTExampleError methodsFor: 'accessing' stamp: 'StefanReichhart 7/3/2016 12:18'!
signaledException: anObject
	signaledException := anObject! !

GTExampleError subclass: #GTExamplePragmaError
	instanceVariableNames: 'pragma'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

GTExamplePragmaError subclass: #GTExampleDeclaredClassMissing
	instanceVariableNames: 'classNamed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleDeclaredClassMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:54'!
classNamed
	^ classNamed! !

!GTExampleDeclaredClassMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:54'!
classNamed: anObject
	classNamed := anObject! !

GTExamplePragmaError subclass: #GTExampleDeclaredMethodMissing
	instanceVariableNames: 'methodClass selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleDeclaredMethodMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:52'!
methodClass
	^ methodClass! !

!GTExampleDeclaredMethodMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:52'!
methodClass: anObject
	methodClass := anObject! !

!GTExampleDeclaredMethodMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:51'!
selector
	^ selector! !

!GTExampleDeclaredMethodMissing methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:51'!
selector: anObject
	selector := anObject! !

!GTExamplePragmaError methodsFor: 'converting' stamp: 'StefanReichhart 7/3/2016 12:22'!
asGTExamplePragmaError
	^ self! !

!GTExamplePragmaError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:31'!
pragma
	^ pragma! !

!GTExamplePragmaError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:31'!
pragma: anObject
	pragma := anObject! !

GTExamplePragmaError subclass: #GTExampleSubjectError
	instanceVariableNames: 'subject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleSubjectError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:53'!
subject
	^ subject! !

!GTExampleSubjectError methodsFor: 'accessing' stamp: 'StefanReichhart 7/2/2016 19:53'!
subject: anObject
	subject := anObject! !

GTExampleError subclass: #GTExampleRecursionDetected
	instanceVariableNames: 'resolved next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
next
	^ next! !

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
next: anObject
	next := anObject! !

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
resolved
	^ resolved! !

!GTExampleRecursionDetected methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 12:00'!
resolved: anObject
	resolved := anObject! !

Error subclass: #GTExampleExpectedExceptionNotRaised
	instanceVariableNames: 'expectedError exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:46'!
exception
	^ exception! !

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 7/13/2015 11:46'!
exception: anObject
	exception := anObject! !

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:48'!
expectedError
	^ expectedError! !

!GTExampleExpectedExceptionNotRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:48'!
expectedError: anObject
	expectedError := anObject! !

Error subclass: #GTExampleUnexpectedExceptionRaised
	instanceVariableNames: 'unexpectedError'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-Exceptions'!

!GTExampleUnexpectedExceptionRaised methodsFor: 'ui' stamp: 'StefanReichhart 4/10/2016 11:14'!
gtDisplayOn: aStream
	super gtDisplayOn: aStream.
	aStream nextPutAll: ' ('.
	self unexpectedError gtDisplayOn: aStream.
	aStream nextPutAll: ')'.! !

!GTExampleUnexpectedExceptionRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:49'!
unexpectedError
	^ unexpectedError! !

!GTExampleUnexpectedExceptionRaised methodsFor: 'accessing' stamp: 'StefanReichhart 8/23/2015 13:49'!
unexpectedError: anObject
	unexpectedError := anObject! !

AbstractMethodIconAction subclass: #GTExampleNautilusAction
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Examples-UI'!

!GTExampleNautilusAction class methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 19:10'!
iconForMethod: aCompiledMethod
	^ MethodWidget methodsIconsCache 
		at: aCompiledMethod 
		ifAbsent: [ self iconForMethod: aCompiledMethod result: nil ]! !

!GTExampleNautilusAction class methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 20:09'!
iconForMethod: aCompiledMethod result: aResult
	aResult
		ifNil: [ ^ Smalltalk ui icons iconNamed: #testNotRunIcon ].
	aResult isSuccess
		ifTrue: [ ^ Smalltalk ui icons iconNamed: #testGreenIcon ].
	aResult isFailure
		ifTrue: [ ^ Smalltalk ui icons iconNamed: #testYellowIcon ].
	aResult isError
		ifTrue: [ ^ Smalltalk ui icons iconNamed: #testRedIcon ].
	^ Smalltalk ui icons iconNamed: #testNotRunIcon! !

!GTExampleNautilusAction class methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 19:00'!
methodChanged: aCompiledMethod
	[	MethodWidget methodsIconsCache removeKey: aCompiledMethod. 
		NautilusUI allSubInstances do: #updateCategoryAndMethodList ]
			on: Error
			do: [ " ignore and continue " ]! !

!GTExampleNautilusAction class methodsFor: 'private' stamp: 'StefanReichhart 7/3/2016 20:09'!
methodUpdated: aCompiledMethod with: aResult
	[	MethodWidget methodsIconsCache 
			at: aCompiledMethod 
			put: (self iconForMethod: aCompiledMethod result: aResult). 
		NautilusUI allSubInstances do: #updateCategoryAndMethodList ]
			on: Error
			do: [ " ignore and continue " ]! !

!GTExampleNautilusAction methodsFor: 'order' stamp: 'StefanReichhart 9/4/2015 17:51'!
actionOrder
	"Return the priority of this action"
	
	^ 0! !

!GTExampleNautilusAction methodsFor: 'order' stamp: 'StefanReichhart 7/3/2016 19:11'!
isActionHandled
	^ method isGTExampleMethod! !

!GTExampleNautilusAction methodsFor: 'order' stamp: 'StefanReichhart 7/3/2016 19:11'!
privateActionIcon
	"Return the icon for this action"
	^ IconicButton new
		target: self;
		actionSelector: #run:;
		arguments: { method };
		labelGraphic: (self class iconForMethod: method);
		color: Color transparent;
		helpText: 'Run';
		extent: 12 @ 12;
		borderWidth: 0! !

!GTExampleNautilusAction methodsFor: 'order' stamp: 'StefanReichhart 7/3/2016 19:55'!
run: aCompiledMethod
	aCompiledMethod gtExample ifNotNil: [ :example | 
		example run isSuccess ifFalse: [ example debug ] ]! !

!RBSelectorEnvironment methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/4/2016 00:08'!
gtBrowseFor: aString
	| browser |
	metaClassSelectors keysAndValuesDo: [ :class :selectors | 
		classSelectors at: class ifAbsentPut: [ IdentitySet new ] ].
	browser := self browse.
	browser ui 
		title: aString; 
		packageSearchUpdate: ''.
	^ browser! !

!TClass methodsFor: '*GT-Examples-UI' stamp: 'TudorGirba 1/26/2016 23:53'!
gtInspectorGTExamplesIn: composite
	<gtInspectorPresentationOrder: 40>
	composite list 
		title: 'Examples';
		display: [ self gtExamples ];
		icon: [:each | each icon];
		format: [:each | String streamContents: [:s | each gtDisplaySummaryOn: s]] ;
		send: [ :each | each returnValue ];
		when: [ self gtExamples notEmpty ]! !

!TClass methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 9/6/2015 16:46'!
gtSpotterGTExamplesFor: aStep
	<spotterOrder: 100>
	aStep listProcessor
		title: 'Examples';
		allCandidates: [ self gtExamples ];
		itemName: [ :example | example gtDisplayString ];
		filter: GTFilterSubstring! !

!Class methodsFor: '*GT-Examples-UI' stamp: 'AliakseiSyrel 5/19/2016 15:03'!
gtInspectorGTExamplesIn: composite
	<gtInspectorPresentationOrder: 40>
	composite list 
		title: 'Examples';
		display: [ self gtExamples ];
		icon: [:each | each icon];
		format: [:each | String streamContents: [:s | each gtDisplaySummaryOn: s]];
		in: [ :list | list sorted: [ :a :b | (list format cull: a) < (list format cull: b)] ];
		send: [ :each | each returnValue ];
		when: [ self gtExamples notEmpty ]! !

!RGMethodDefinition methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 7/2/2016 19:44'!
asGTExampleSubject
	^ GTExampleMethodSubject new 
		subjectClass: self realClass;
		subjectSelector: self selector;
		yourself! !

!Pragma methodsFor: '*GT-Examples-Core' stamp: 'StefanReichhart 9/9/2015 19:54'!
isGTExamplePragma
	<toBeDeprecated: #gtExample:>
	^ self keyword = #gtExample
		or: [ self keyword = #gtExample: ]! !

!AbstractNautilusUI class methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/3/2016 17:36'!
examplesClassMenu: aBuilder
	<contextMenu>
	<nautilusGlobalClassMenu>
	^ GTExampleNautilus new
		onClass: aBuilder;
		yourself! !

!AbstractNautilusUI class methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/3/2016 17:34'!
examplesMethodMenu: aBuilder
	<contextMenu>
	<nautilusGlobalMethodMenu>
	^ GTExampleNautilus new
		onCompiledMethod: aBuilder;
		yourself! !

!AbstractNautilusUI class methodsFor: '*GT-Examples-UI' stamp: 'StefanReichhart 7/3/2016 17:34'!
examplesPackageMenu: aBuilder
	<contextMenu>
	<nautilusGlobalPackageMenu>
	^ GTExampleNautilus new
		onPackage: aBuilder;
		yourself! !
